<?php


namespace Fastrack\Location;


use Ballen\Distical\Calculator;
use Ballen\Distical\Entities\LatLong;
use Fastrack\Property\Property;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use Psr\Log\LoggerInterface;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use UndefinedOffset\SortableGridField\Forms\GridFieldSortableRows;

/**
 * @method ManyManyList Properties
 * @method string getAddressFormatted()
 * @method ManyManyList Categories()
 * @method Region Region()
 * @method SubRegion SubRegion()
 * @method Imgix MapPin()
 * @method Imgix Image()
 * @property string $Name
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @property string $Website
 * @property string $Phone
 * @property string $Email
 * @property string $Address1
 * @property string $Address2
 * @property string $City
 * @property string $Postcode
 * @property string $State
 * @property string $Country
 * @property string $Directions
 * @property string $Latitude
 * @property string $Longitude
 * @property string $MapUrl
 */
class Place extends DataObject implements PermissionProvider
{

	private static string $table_name = 'place';

	private static array $db = [
		'Name' => 'Varchar',
		'Website' => 'Varchar',
		'Phone' => 'Varchar',
		'Email' => 'Varchar',
	];

	private static array $many_many = [
		'Properties' => Property::class,
		'Categories' => PlaceCategory::class,
	];

	private static array $translate = [
		'Name',
	];

	private static array $indexes = [
		'SlugIndex' => [
			'type' => 'unique',
			'columns' => ['Slug'],
		],
	];

	private static bool $auto_relate_places = true;

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$fields->removeByName('Properties');

		$categoryConfig = GridFieldConfig_RelationEditor::create(20);
		$categoryConfig->removeComponentsByType(GridFieldAddNewButton::class);
		$categoryConfig->addComponent(new GridFieldAddExistingSearchButton());
		$categoriesGrid = GridField::create('Categories', 'Categories', $this->Categories(), $categoryConfig);
		$fields->replaceField('Categories', $categoriesGrid);

		ModelUtilities::hideConfigFields($this, $fields);
		return $fields;
	}

	public function onBeforeWrite()
	{
		$this->Title = $this->Name;
		$this->addPropertiesWithinRadius();
		parent::onBeforeWrite();
	}

	private function addPropertiesWithinRadius()
	{
		if (!Config::inst()->get(Place::class, 'auto_relate_places')) {
			return;
		}
		if ($this->Latitude && $this->Longitude) {
			$placeCoordinates = new LatLong(floatval($this->Latitude), floatval($this->Longitude));
			$properties = Property::get();
			foreach ($properties as $property) {
				$this->Properties()->remove($property);
				if ($property->Latitude && $property->Longitude) {
					try {
						$propertyCoordinates = new LatLong(floatval($property->Latitude), floatval($property->Longitude));
						$distance = new Calculator($propertyCoordinates, $placeCoordinates);
						$radius = $property->PlaceRadius ?: 50;
						if ($distance->get()->asKilometres() <= $radius) {
							$this->Properties()->add($property);
						}
					} catch (\Exception $e) {
						Injector::inst()->get(LoggerInterface::class)->info($e->getMessage());
					}
				}
			}
		}
	}

	public function providePermissions(): array
	{
		$permissions = [
			'MANAGE_PLACES' => [
				'name' => 'Manage places',
				'category' => 'Places',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_PLACES']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_PLACES']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_PLACES']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_PLACES']);
	}
}
